@@ -22,19 +22,22 @@ module REXML
 
 		##
 		# Constructor
-		# @param parent if given, is set as the parent of this object
+		# @param parent If first is String, it must be String and set to
+		# external_id. Otherwise, it is set as the parent of this object.
 		# @param first can be multiple types.  If String, name is set to this
 		# and external_id is set to nil.  If DocType, the object is cloned.  If
 		# Source, the source is scanned for the DOCTYPE declaration.
 		def initialize( first, parent=nil )
-			super(parent)
 			if first.kind_of? String
+				super()
 				@name = first
-				@external_id = nil
+				@external_id = parent
 			elsif first.kind_of? DocType
+				super(parent)
 				@name = first.name
 				@external_id = first.external_id
 			elsif first.kind_of? Source
+				super(parent)
 				md = first.match( PATTERN_RE, true )
 				identity = md[1]
 				close = md[2]
@@ -120,8 +123,7 @@ module REXML
 		def DocType.parser source
 			begin
 				md = source.match(/\s*(.*?)>/um)
-				nxt = md[1]
-				until md[1] == "]" 
+				until md[1].strip == "]" 
 					case md[1]
 					when /^%/ #/u
 						md = source.match(/^\s*%(.*?);/um, true)
@@ -142,13 +144,13 @@ module REXML
 						if md.nil?
 							raise "no match!"
 						else
-							raise "illegal entry \"#{md[1]}\" in DOCTYPE"
+							raise "illegal entry \"#{md[1]}\" in DOCTYPE\n(match data was '#{md[0]}'"
 						end
 					end
 					md = source.match(/\s*(.*?)>/um)
 					raise ParseException.new( "Invalid end of DOCTYPE declaration \"#{source.buffer}\"", source ) if md.nil?
 				end
-				source.match(/\s*]>/um, true)
+				source.match(/\s*]\s*>/um, true)
 			rescue ParseException
 				raise
 			rescue Exception => err
@@ -262,6 +264,7 @@ module REXML
 			rv = "<!ENTITY #@name "
 			rv << "#@middle " if @middle.size > 0
 			rv << @content
+			rv << ">"
 			rv
 		end
 
@@ -301,7 +304,7 @@ module REXML
 			"<!NOTATION #@name #@middle #@rest>"
 		end
 
-		def write( output, indent )
+		def write( output, indent=-1 )
 			output << ('   '*indent) if indent > 0
 			output << to_s
 		end
