@@ -125,10 +125,10 @@ module REXML
 		# document will be written
 		# @param indent (optional) if given, the starting indent for the lines
 		# in the document.
-		def write( output, indent=0 )
+		def write( output, indent=-1 )
 			@children.each { |node|
 				node.write( output, indent )
-				output << "\n" unless node == @children[-1]
+				output << "\n" unless indent<0 or node == @children[-1]
 			}
 		end
 
@@ -150,10 +150,10 @@ module REXML
 		def Document.parse_stream( source, listener )
 			if			source.kind_of? Source
 				# do nothing
-			elsif		source.kind_of?  File
+			elsif		source.kind_of?  IO
 				source = IOSource.new(source)
 			elsif		source.kind_of? String
-				source = Source.new xmlSrc
+				source = Source.new source
 			else
 				raise "Unknown source type!"
 			end
@@ -185,30 +185,39 @@ module REXML
 		# This and parse_stream could have been combined, but separating them
 		# improves the speed of REXML
 		def parse( source )
-			while not source.empty?
-				source.match( /^\s*/um, true )
-				word = source.match( /^(<.*?)>/um )
-				word = word[1] unless word.nil?
-				case word
-				when nil
-					word = source.match( /\s*(\S+)/um, true )
-					return if word.nil?
-					raise ParseException.new( "data found outside of root element (data is '#{word}')", source ) if word[0].strip.length > 0
-				when Comment::START_RE
-					self.add( Comment.new( source ) )
-				when DocType::START_RE
-					self.add( DocType.new( source ) )
-				when XMLDecl::START_RE
-					self.add( XMLDecl.new( source ) )
-				when Instruction::START_RE
-					self.add( Instruction.new( source ) )
-				else
-					Element.new( source, self, @context )
+			begin 
+				while not source.empty?
+					source.match( /^\s*/um, true )
+					word = source.match( /^(<.*?)>/um )
+					word = word[1] unless word.nil?
+					case word
+					when nil
+						word = source.match( /\s*(\S+)/um, true )
+						return if word.nil?
+						raise ParseException.new( "data found outside of root element (data is '#{word}')", source ) if word[0].strip.length > 0
+					when Comment::START_RE
+						self.add( Comment.new( source ) )
+					when DocType::START_RE
+						self.add( DocType.new( source ) )
+					when XMLDecl::START_RE
+						self.add( XMLDecl.new( source ) )
+					when Instruction::START_RE
+						self.add( Instruction.new( source ) )
+					else
+						Element.new( source, self, @context )
+					end
 				end
-			end
-			unless @elements.size == 1
-				#@children.find_all{|x| puts x if x.kind_of? Element }
-				raise "the document does not have exactly one root"
+				unless @elements.size == 1
+					#@children.find_all{|x| puts x if x.kind_of? Element }
+					raise "the document does not have exactly one root"
+				end
+			rescue ParseException
+				$!.source = source
+				$!.element = self
+				raise
+			rescue Exception
+				old_ex = $!
+				raise ParseException.new("unidentified error", source, self, old_ex)
 			end
 		end
 	end
