@@ -20,14 +20,28 @@ module REXML
 		attr_accessor :context
 
 		# Constructor
-		# @param parent if supplied, must be a Parent, and will be used as
-		# the parent of this object.
-		# @param arg if not supplied, will be set to the default value.
-		# If a String, the name of this object will be set to the argument.
-		# If an Element, the object will be shallowly cloned; name, attributes,
-		# and namespaces will be copied.
-		# If a Source, the source will be scanned and parsed for an Element,
-		# and all child elements will be recursively parsed as well.
+		# arg:: 
+		# 	if not supplied, will be set to the default value.
+		# 	If a String, the name of this object will be set to the argument.
+		# 	If an Element, the object will be shallowly cloned; name, 
+		# 	attributes, and namespaces will be copied.
+		# 	If a Source, the source will be scanned and parsed for an Element,
+		# 	and all child elements will be recursively parsed as well.
+		# parent:: 
+		# 	if supplied, must be a Parent, and will be used as
+		# 	the parent of this object.
+		# context::
+		# 	If supplied, must be a hash containing context items.  Context items
+		# 	include:
+		# 		* :respect_whitespace:: the value of this is +:all+ or an array of
+		# 			strings being the names of the elements to respect
+		# 			whitespace for.  Defaults to +:all+.
+		# 		* :ignore_whitespace:: the value can be +:all+ or an array of
+		# 			strings being the names of the elements to ignore whitespace on.
+		# 			Overrides :respect_whitespace.
+		# 		* :raw:: can be +:all+, or an array of strings being the names of
+		# 			the elements to process in raw mode.  In raw mode, special
+		# 			characters in text is not converted to or from entities.
 		def initialize( arg = UNDEFINED, parent=nil, context=nil )
 			super(parent)
 
@@ -48,26 +62,49 @@ module REXML
 			end
 		end
 
+		# Creates a shallow copy of self.
+		#   d = Document.new "<a><b/><b/><c><d/></c></a>"
+		#   new_a = d.root.clone
+		#   puts new_a  # => "<a/>"
 		def clone
 			Element.new self
 		end
 
-		# @return the root of the document that this element belongs to.
-		# If this element doesn't belong to a document, but does belong to
-		# another Element, the parent's root will be returned, until the
+		# Evaluates to the root element of the document that this element 
+		# belongs to. If this element doesn't belong to a document, but does
+		# belong to another Element, the parent's root will be returned, until the
 		# earliest ancestor is found.
 		def root
 			parent.nil? ? self : parent.root
 		end
 
+		# Evaluates to +true+ if whitespace is respected for this element.  This
+		# is the case if the context has +:respect_whitespace+ set to +:all+ or
+		# an array containing the name of this element, and +:ignore_whitespace+
+		# isn't set to +:all+ or an array containing the name of this element.
+		# The evaluation is tested against +expanded_name+, and so is namespace
+		# sensitive.
 		def whitespace
-			@whitespace = (@context and @context[:respect_whitespace] and
-				(@context[:respect_whitespace] == :all or
-				 @context[:respect_whitespace].include? expanded_name)
-			)
+			@whitespace = nil
+			if @context
+				@whitespace = (@context[:respect_whitespace] and
+					(@context[:respect_whitespace] == :all or
+					 @context[:respect_whitespace].include? expanded_name)
+				)
+				@whitespace = (@context[:compress_whitespace] and
+					(@context[:compress_whitespace] == :all or
+					 @context[:compress_whitespace].include? expanded_name)
+				)
+			end
+			@whitespace = true unless @whitespace == false
 			@whitespace
 		end
 
+		# Evaluates to +true+ if raw mode is set for this element.  This
+		# is the case if the context has +:raw+ set to +:all+ or
+		# an array containing the name of this element.
+		# The evaluation is tested against +expanded_name+, and so is namespace
+		# sensitive.
 		def raw
 			@raw = (@context and @context[:raw] and
 			(@context[:raw] == :all or
@@ -81,7 +118,8 @@ module REXML
 		# Namespaces                                    #
 		#################################################
 
-		# @return an Array of all defined namespaces
+		# Evaluates to an +Array+ containing the prefixes (names) of all defined
+		# namespaces
 		def prefixes
 			prefixes = []
 			prefixes = parent.prefixes if parent
@@ -89,8 +127,9 @@ module REXML
 			return prefixes
 		end
 
-		# @return the URI for a namespace, or the empty string if no such 
-		# namespace is declared
+		# Evalutas to the URI for a prefix, or the empty string if no such 
+		# namespace is declared for this element. Evaluates recursively for
+		# ancestors.
 		def namespace prefix=nil
 			if prefix.nil?
 				prefix = prefix()
@@ -101,11 +140,11 @@ module REXML
 			return ns
 		end
 
-		# Adds a namespace to this element
-		# @param prefix the namespace.  If the second argument is not supplied,
-		# this should be the URI of the default namespace.
-		# @param value (optional) the URI of the namespace.  If "", then unsets
-		# the default namespace
+		# Adds a namespace to this element.
+		#		a = Element.new("a")
+		#		a.add_namespace("foo", "bar")
+		#		a.add_namespace(nil, "twiddle")
+		#		puts a   #-> <a xmlns:foo='bar' xmlns='twiddle'/>
 		def add_namespace( prefix, uri=nil )
 			unless uri
 				@attributes["xmlns"] = prefix
@@ -347,7 +386,7 @@ module REXML
 		# @param writer A String or IO (or any object supporting <<(String))
 		# @param indent if supplied, must be an Integer, which is then used
 		# to determine the indentation of this element and its children.
-		def write(writer, indent=0)
+		def write(writer, indent=-1)
 			indent writer, indent
 			writer << "<#@expanded_name"
 
@@ -491,7 +530,7 @@ module REXML
 
 				# A private helper method
 				def write_children( writer, indent )
-					cr = indent < 0 ? "" : "\n"
+					cr = (indent < 0 or whitespace) ? "" : "\n"
 					#if size == 1 and @children[0].kind_of?(Text)
 					#	self[0].write( writer, -1 )
 					if indent == -1
@@ -616,6 +655,7 @@ module REXML
 		# new, default Element will be constructed
 		# @return the added Element
 		def add element=nil
+			rv = nil
 			if element.nil?
 				Element.new "", self, @element.context
 			elsif not element.kind_of?(Element)
@@ -623,6 +663,7 @@ module REXML
 			else
 				@element << element
 				element.context = @element.context
+				element
 			end
 		end
 
