@@ -10,7 +10,7 @@ module REXML
 		def name=( name )
 			@expanded_name = name
 			if name.include? ":"
-				name =~ /(\w+):(\w+)/
+				name =~ /(\w+):(\w+)/u
 				@prefix = $1
 				@name = $2
 			else
