@@ -1,11 +1,15 @@
+require "rexml/parseexception"
+
 module REXML
 	# Represents a node in the tree.  Nodes are never encountered except as
 	# superclasses of other objects.  Nodes have siblings.
 	module Node
 		# The order in which the substitutions occur
-		SPECIALS = [ /&/, /</, />/ ]
+		SPECIALS = [ /&(?!#?[\w-]+;)/u, /</u, />/u, /"/u, /'/u ]
+		SLAICEPS = [ "&", "<", ">", '"', "'" ]
 		# Characters which are substituted in written strings
-		SUBSTITUTES = ['&amp;', '&lt;', '&gt;']
+		SUBSTITUTES = ['&amp;', '&lt;', '&gt;', '&quot;', '&apos;']
+		SETUTITSBUS = [/&amp;/u, /&lt;/u, /&gt;/u, /&quot;/u, /&apos;/u]
 
 		# @return the next sibling (nil if unset)
 		def next_sibling_node
@@ -21,7 +25,7 @@ module REXML
 			@parent[ ind - 1 ]
 		end
 
-		def to_s indent=0
+		def to_s indent=-1
 			rv = ""
 			write rv,indent
 			rv
@@ -40,7 +44,26 @@ module REXML
 			copy.gsub!( SPECIALS[0], SUBSTITUTES[0] )
 			copy.gsub!( SPECIALS[1], SUBSTITUTES[1] )
 			copy.gsub!( SPECIALS[2], SUBSTITUTES[2] )
+			copy.gsub!( SPECIALS[3], SUBSTITUTES[3] )
+			copy.gsub!( SPECIALS[4], SUBSTITUTES[4] )
 			out << copy
 		end
+
+		# Reads text, substituting entities
+		def read_with_substitution( input, illegal=nil )
+			copy = input.clone
+
+			if copy =~ illegal
+				raise ParseException.new( "malformed text: Illegal character #$& in \"#{copy}\"" )
+			end if illegal
+			
+			copy.gsub!( SETUTITSBUS[0], SLAICEPS[0] )
+			copy.gsub!( SETUTITSBUS[1], SLAICEPS[1] )
+			copy.gsub!( SETUTITSBUS[2], SLAICEPS[2] )
+			copy.gsub!( SETUTITSBUS[3], SLAICEPS[3] )
+			copy.gsub!( SETUTITSBUS[4], SLAICEPS[4] )
+
+			copy
+		end
 	end
 end
