@@ -54,8 +54,7 @@ module REXML
 		end
 
 		# @param other a String or a Text
-		# @return true if the content of this object matches the supplied 
-		# argument's content
+		# @return the result of (to_s <=> arg.to_s)
 		def <=>( other )
 			@string <=> other.to_s
 		end
@@ -64,7 +63,7 @@ module REXML
 		  @string
 		end
 
-		def write( writer, indent=0 ) 
+		def write( writer, indent=-1 ) 
 			indent( writer, indent )
 			if @raw
 				writer << @string
